# # from inspect import currentframe, getframeinfo

# # dtype = int


# # class VarHist:
# # 	def __init__(self, data, dtype):
# # 		class Obj(dtype):
# # 			def __init__(self, data: any):
# # 				self.data: dtype = data
# # 				# self


# # 			def __setattr__(self, name, value):
# # 				frame_info = getframeinfo(currentframe()).filename + ':' + str(getframeinfo(currentframe()).lineno)
# # 				print(f"{frame_info}: Attribute \'{name}\' changed to {value}")
# # 				super().__setattr__(name, value)



# # 		self.object = Obj(data);

# # data = VarHist(1, dtype).object

# # print(type(data))

# import sys
# from module import varhist as VarHist

# VarHist.track('Obj.x.val', 'c')
 
# sys.settrace(VarHist.trace)
 
# def main():

# 	c = 314124
# 	c -= 12

# 	a = 3

# 	c = a
# 	class Bob:
# 		def __init__(self, val):
# 			self.val = val
# 	class John:
# 		x = Bob(4)

# 	Obj = John()
# 	Obj.x.val = 2
# 	# Obj.x.vadl = 3
# 	Obj.x.val -=21

# 	# assert VarHist.HIST == [[3, 42]]
# 	# a = 10
# 	# a = [20]
 
# 	# for i in range(5):
# 	# 	c = i

# main()
# VarHist.history('Obj.x.val', 'c')

import sys
from module import varhist

sys.settrace(varhist.trace)
varhist.track('j')

class Person:
	def __init__(self, name):
		j = 2
		self.name = name

def main():
    varhist.track('x', 'y')
    somebody = Person("Rithwik")
    # Modify the name
    somebody.name = "John"
    somebody.name += ' Green'
    
main()
varhist.history('j')
# print(varhist.HIST['c'])


import traceback, sys

class varhist:

	HIST = {}
	checking = []

	line = 0

	@classmethod
	def trace(self, frame, event, arg):
		line = self.line
		self.line = frame.f_lineno #traceback.format_stack()[-2].split(',')[1].strip(' ').split(' ')[1]
		dictionary = frame.f_locals
		# dictionary.update(frame.f_globals)
		# dictionary.update(frame.f_builtins)
		for name in self.checking:
			if name[0] in dictionary:
				val = dictionary[name[0]]
				try:
					for sub in name[1:]:
						val = getattr(val, sub)

				except AttributeError:
					continue

				if '.'.join(name) in self.HIST:
					if val != self.HIST['.'.join(name)][-1][0]:
						self.HIST['.'.join(name)].append([val, line, frame.f_code.co_name])

				else:
					self.HIST['.'.join(name)] = [[val, line, frame.f_code.co_name]]

		# for name,val in dictionary.items():
		# 	if name in self.checking:
		# 		if name in self.HIST:
		# 			if val != self.HIST[name][-1][0]:
		# 				self.HIST[name].append([val, line, frame.f_code.co_name])

		# 		else:
		# 			self.HIST[name] = [[val, line, frame.f_code.co_name]]
		return self.trace


	@classmethod
	def track(self, *var_names):
		for var_name in var_names:
			self.checking.append(var_name.split('.'))

	@classmethod
	def history(self, *var_names):
		br = False
		for var_name in var_names:
			if br:
				print('\n')
			if var_name not in self.HIST and var_name.split('.') not in self.checking:
				raise ValueError(f'Variable is not being tracked. Use VarHist.track({var_name}).')

			elif var_name.split('.') in self.checking and var_name not in self.HIST:
				print(f'Variable \'{var_name}\' was never created.')
			else:
				print(f'----------Start of History for \'{var_name}\'----------')
				for value in self.HIST[var_name]:
					print(f"Variable '{var_name}' changed to '{value[0]}' on line '{value[1]}' in function '{value[2]}'")

				print(f'----------End of History for \'{var_name}\'----------')

			br = True





